/*
 *  Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.wso2.ballerinalang.compiler.tree.clauses;

import org.ballerinalang.model.tree.NodeKind;
import org.ballerinalang.model.tree.clauses.OrderByVariableNode;
import org.ballerinalang.model.tree.expressions.ExpressionNode;
import org.wso2.ballerinalang.compiler.tree.BLangNode;
import org.wso2.ballerinalang.compiler.tree.BLangNodeVisitor;

/**
 * @see OrderByVariableNode
 * @since 0.965.0
 *
 * Implementation of {@link OrderByVariableNode}.
 */
public class BLangOrderByVariable extends BLangNode implements OrderByVariableNode {

    private ExpressionNode varRef;
    private String orderByType;

    @Override
    public void setVariableReference(ExpressionNode varRef) {
        this.varRef = varRef;
    }

    @Override
    public ExpressionNode getVariableReference() {
        return varRef;
    }

    @Override
    public void setOrderByType(boolean isAscending, boolean isDescending) {
        if (isAscending) {
            this.orderByType = "asc";
        } else if (isDescending) {
            this.orderByType = "desc";
        } else {
            this.orderByType = "";
        }
    }

    @Override
    public String getOrderByType() {
        return orderByType;
    }

    @Override
    public void accept(BLangNodeVisitor visitor) {
        visitor.visit(this);
    }

    @Override
    public NodeKind getKind() {
        return NodeKind.ORDER_BY_VARIABLE;
    }
}
